/* eslint-disable no-console, no-process-exit */

const chalk = require('chalk');
const inquirer = require('inquirer');
const commander = require('commander');
const fs = require('fs-extra');
const path = require('path');
const spawn = require('cross-spawn');
const os = require('os');
const packageJson = require('../package.json');

let projectName;

const program = new commander.Command(packageJson.name)
  .version(packageJson.version)
  .arguments('<extension-folder>')
  .usage(`${chalk.green('<extension-folder>')} [options]`)
  .action(name => {
    projectName = name;
  })
  .option('--verbose', 'print additional logs')
  .parse(process.argv);

if (typeof projectName === 'undefined') {
  console.error('Please specify the extension folder:');
  console.log(`  ${chalk.cyan(program.name())} ${chalk.green('<extension-folder>')}`);
  console.log();
  console.log('For example:');
  console.log(`  ${chalk.cyan(program.name())} ${chalk.green('my-contentful-extension')}`);
  console.log();
  console.log(`Run ${chalk.cyan(`${program.name()} --help`)} to see all options.`);
  process.exit(1);
}

function install(root, dependencies, verbose, isDev) {
  return new Promise((resolve, reject) => {
    let command;
    let args;
    command = 'npm';
    args = [
      'install',
      '--prefix',
      root,
      isDev ? '--save-dev' : '--save',
      '--save-exact',
      '--loglevel',
      'error'
    ].concat(dependencies);

    if (verbose) {
      args.push('--verbose');
    }

    const child = spawn(command, args, { stdio: 'inherit' });
    child.on('close', code => {
      if (code !== 0) {
        reject({
          command: `${command} ${args.join(' ')}`
        });
        return;
      }
      resolve();
    });
  });
}

function run(root, payload, verbose, originalDirectory) {
  const devDependencies =
    process.env.LOCAL_TEST === 'true'
      ? [
          path.resolve('./packages/contentful-extension-scripts'),
          path.resolve('./packages/eslint-config-extension')
        ]
      : ['@contentful/contentful-extension-scripts', '@contentful/eslint-config-extension'];

  return install(root, devDependencies, verbose, true).then(() => {
    const init = require(`${root}/node_modules/@contentful/contentful-extension-scripts/scripts/init.js`);
    init(root, payload, originalDirectory);
  });
}

const createExtension = async (name, verbose) => {
  const root = path.resolve(name);
  const appName = path.basename(root);

  if (fs.pathExistsSync(name)) {
    console.error(chalk.red(`${root} folder already exists.`));
    process.exit(0);
  }
  fs.ensureDirSync(name);

  console.log(`Creating a new Contentful extension in ${chalk.green(root)}.`);
  console.log();

  const answers = await inquirer.prompt([
    {
      type: 'list',
      name: 'type',
      message: 'Select type of extension:',
      choices: [
        { name: 'Field extension', value: 'fields' },
        { name: 'Sidebar extension', value: 'sidebar' },
        { name: 'Entry editor extension', value: 'entry' },
        { name: 'Page extension', value: 'page' }
      ]
    },
    {
      type: 'checkbox',
      name: 'fields',
      message: 'Select field types for this extension:',
      choices: [
        { name: 'Symbol' },
        { name: 'Symbols' },
        { name: 'Text' },
        { name: 'RichText' },
        { name: 'Integer' },
        { name: 'Number' },
        { name: 'Date' },
        { name: 'Boolean' },
        { name: 'Object' },
        { name: 'Entry' },
        { name: 'Entries' },
        { name: 'Asset' },
        { name: 'Assets' }
      ],
      when: function(answers) {
        return answers.type === 'fields';
      },
      validate: function(answer) {
        if (answer.length < 1) {
          return 'You must choose at least one field type.';
        }
        return true;
      }
    },
    {
      type: 'list',
      name: 'language',
      message: 'What language you want to use to develop extension:',
      choices: [
        { name: 'JavaScript', value: 'javascript' },
        { name: 'TypeScript', value: 'typescript' }
      ]
    }
  ]);

  fs.writeFileSync(
    path.join(root, 'package.json'),
    JSON.stringify(
      {
        name: appName,
        version: '0.1.0',
        private: true
      },
      null,
      2
    ) + os.EOL
  );

  const originalDirectory = process.cwd();
  run(
    root,
    {
      name: appName,
      type: answers.type,
      fields: answers.fields || [],
      language: answers.language
    },
    verbose,
    originalDirectory
  );
};

createExtension(projectName, program.verbose);
